import Make from "@rbxts/make"
import { World, useEvent } from "@rbxts/matter"
import { Model, PlayerCharacter } from "ReplicatedStorage/ecs/components"

/**
 * When players die, disconnect all Motor6D instances.
 * 
 * The Motor6D instances are replaced with BallSocketContraints.
 */
function playersRagdollOnDeath(world: World): void {
    for (const [_, playerCharacter, model] of world.query(PlayerCharacter, Model)) {
        if (!model.model || !playerCharacter.character) continue

        playerCharacter.humanoid.BreakJointsOnDeath = false

        for (const [_] of useEvent(playerCharacter.humanoid, "Died")) {
            model.model.GetDescendants().forEach((v, _) => {
                if (v.IsA("Motor6D")) {
                    const attachment0 = Make("Attachment", {
                        CFrame: v.C0,
                        Parent: v.Part0
                    })
                    const attachment1 = Make("Attachment", {
                        CFrame: v.C1,
                        Parent: v.Part1
                    })

                    Make("BallSocketConstraint", {
                        Attachment0: attachment0,
                        Attachment1: attachment1,
                        LimitsEnabled: true,
                        TwistLimitsEnabled: true,
                        Parent: v.Parent
                    })

                    v.Destroy()
                }
            })
            
            // Makes it so their head doesn't just dangle
            playerCharacter.character.HumanoidRootPart.ApplyImpulse(Vector3.FromNormalId(Enum.NormalId.Back).mul(100))
        }
    }

    for (const [id] of world.query(PlayerCharacter).without(Model)) {
        world.despawn(id)
    }
}

export = playersRagdollOnDeath